var vows = require("vows"),
    assert = require("assert"),
    orm = require("../lib/orm");

module.exports = {
	start: function (description, url) {
		vows.describe(description).addBatch({
			"Connecting to database": {
				topic: function () {
					orm.connect(url, (function (cb) {
						return function (success, db) {
							cb(null, { success: success, db: db });
						};
					})(this.callback));
				},
				"Connected": function (topic) {
					assert.isBoolean(topic.success);
					assert.strictEqual(topic.success, true);
				},
				"Define model": {
					topic: function (topic) {
						return topic.db.define("folder", {
							created: Date,
							   name: String,
							   size: Number
						});
					},
					"Has common methods": function (model) {
						assert.isFunction(model);
						assert.isFunction(model.hasOne);
						assert.isFunction(model.hasMany);
						assert.isFunction(model.get);
						assert.isFunction(model.find);
						assert.isFunction(model.sync);
						assert.isFunction(model.clear);
					},
					"Add relationships": {
						topic: function (model) {
							model.hasOne("parent");

							return model;
						},
						"Sync": {
							topic: function (model) {
								model.sync();

								return model;
							},
							"Create a record": function (model) {
								console.log(model);
							}
						}
					}
				}
			}
		}).run();
	}
};